﻿using System;
using System.Data;
using System.Data.OleDb;

using DevExpress.Utils;
using DevExpress.XtraEditors;
using DevExpress.XtraGrid.Views.Grid;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            #region 데이터를 로드한다.

            DataSet dataSet = new DataSet();

            OleDbConnection connection = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=nwind.mdb");

            OleDbDataAdapter customersAdapter = new OleDbDataAdapter("SELECT * FROM Customers", connection);

            OleDbDataAdapter ordersAdapter = new OleDbDataAdapter("SELECT * FROM Orders", connection);

            OleDbDataAdapter orderDetailsAdapter = new OleDbDataAdapter("SELECT * FROM [Order Details]", connection);

            customersAdapter.Fill(dataSet, "Customers");

            ordersAdapter.Fill(dataSet, "Orders");

            orderDetailsAdapter.Fill(dataSet, "Order Details");

            DataColumn customerIDKeyColumn        = dataSet.Tables["Customers"].Columns["CustomerID"];
            DataColumn customerIDForeignKeyColumn = dataSet.Tables["Orders"   ].Columns["CustomerID"];

            dataSet.Relations.Add("CustomersOrders", customerIDKeyColumn, customerIDForeignKeyColumn);

            DataColumn orderIDKeyColumn        = dataSet.Tables["Orders"       ].Columns["OrderID"];
            DataColumn orderIDForeignKeyColumn = dataSet.Tables["Order Details"].Columns["OrderID"];

            dataSet.Relations.Add("OrdersOrderDatails", orderIDKeyColumn, orderIDForeignKeyColumn);

            #endregion
            #region 조부모 그리드 뷰를 설정한다.

            this.gridView1.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            this.gridView1.OptionsDetail.EnableMasterViewMode = true;

            this.gridView1.OptionsBehavior.AutoPopulateColumns = true;

            #endregion
            #region 부모 그리드 뷰를 설정한다.

            this.gridView2.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            this.gridView2.OptionsBehavior.AutoPopulateColumns = true;

            #endregion
            #region 자식 그리드 뷰를 설정한다.

            this.gridView3.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            this.gridView3.OptionsBehavior.AutoPopulateColumns = true;

            #endregion

            this.gridControl.DataSource = dataSet;
            this.gridControl.DataMember = "Customers";

            this.gridView1.BestFitColumns();

            this.gridView1.MasterRowExpanded += gridView_MasterRowExpanded;
            this.executeButton.Click         += executeButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 그리드 뷰 마스터 행 확장시 처리하기 - gridView_MasterRowExpanded(sender, e)

        /// <summary>
        /// 그리드 뷰 마스터 행 확장시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_MasterRowExpanded(object sender, CustomMasterRowEventArgs e)
        {
            GridView gridView = sender as GridView;

            GridView childGridView = gridView.GetDetailView(e.RowHandle, e.RelationIndex) as GridView;

            if(childGridView.Name == "gridView1")
            {
                childGridView.Name = "gridView2";

                childGridView.BestFitColumns();

                childGridView.MasterRowExpanded += gridView_MasterRowExpanded;
            }
            else if(childGridView.Name == "gridView2")
            {
                childGridView.Name = "gridView3";

                childGridView.BestFitColumns();
            }
        }

        #endregion
        #region 실행 버튼 클릭시 처리하기 - executeButton_Click(sender, e)

        /// <summary>
        /// 실행 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void executeButton_Click(object sender, EventArgs e)
        {
            ExpandRow(this.gridView1, this.gridView1.FocusedRowHandle);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 행 확장하기 - ExpandRow(gridView, rowHandle)

        /// <summary>
        /// 행 확장하기
        /// </summary>
        /// <param name="gridView">그리드 뷰</param>
        /// <param name="rowHandle">행 핸들</param>
        private void ExpandRow(GridView gridView, int rowHandle)
        {
            int relationCount = gridView.GetRelationCount(rowHandle);

            for(int index = relationCount - 1; index >= 0; index--)
            {
                gridView.ExpandMasterRow(rowHandle, index);

                GridView childView = gridView.GetDetailView(rowHandle, index) as GridView;

                if(childView != null)
                {
                    int childRowCount = childView.DataRowCount;

                    for(var handle = 0; handle < childRowCount; handle++)
                    {
                        ExpandRow(childView, handle);
                    }
                }
            }
        }

        #endregion
    }
}